"""
Zurich Instruments LabOne Python API Example

Demonstrate how to connect to a Zurich Instruments device via the Data Server
and perform some basic device configuration.
"""

# Copyright 2014 Zurich Instruments AG

import time
import re
import numpy as np

import zhinst.utils


def run_example(port=None, api_level=None, amplitude=0.100):
    """
    Run the example: Connect to a Zurich Instruments device via the Data Server,
    configure some settings on the instrument, obtain a single demodulator
    sample via getSample(), calculate and add its RMS amplitude as a field to
    the ``sample`` dictionary and return that dictionary.

    Note:

      This is intended to be a simple example demonstrating how to connect to a
      Zurich Instruments device from ziPython. In most cases, data acquisition
      should use either ziDAQServer's poll() method or an instance of the
      ziDAQRecorder class, not the getSample() method.

    Requirements:

      Hardware configuration: Connect signal output 1 to signal input 1 with a
      BNC cable.

    Arguments:

      port (int, optional): The port to use when connecting to the Data Server
        (the HF2 Data Server uses port 8005, the UHF Data Server uses port
        8004).

      api_level (int, optional): The API level ziPython should use (either 1 or
        4; HF2 only supports Level 1, Level 4 is recommended for UHF).

      amplitude (float, optional): The amplitude to set on the signal output.

    Returns:

      sample (dict): The demodulator sample dictionary.
    """
    # Create a connection to a Zurich Instruments Data Server
    daq = zhinst.utils.autoConnect(port, api_level)
    # Note: to specify directly which server you would like to connect to use
    # the following with port=8005 for HF2 and port=8004 for UHF:
    # daq = ziDAQServer('localhost',port)

    # Detect device
    device = zhinst.utils.autoDetect(daq)

    # Find out whether the device is an HF2 or a UHF
    devtype = daq.getByte('/%s/features/devtype' % device)
    options = daq.getByte('/%s/features/options' % device)

    # Configure the device
    out_c = 0
    in_c = 0
    demod_c = 0
    osc_c = 0
    demod_rate = 10e3
    time_constant = 1e-6
    if re.match('UHF', devtype) and (not re.match('MF', options)):
        out_mixer_c = 3
    elif re.match('HF2', devtype) and (not re.match('MF', options)):
        out_mixer_c = 6
    else:
        out_mixer_c = 0

    # Create a base configuration: disable all outputs, demods and scopes
    general_setting = [['/%s/demods/*/rate' % device, 0],
                       ['/%s/demods/*/trigger' % device, 0],
                       ['/%s/sigouts/*/enables/*' % device, 0]]
    if re.match('HF2', devtype):
        general_setting.append(['/%s/scopes/*/trigchannel' % device, -1])
    else:  # UHFLI
        general_setting.append(['/%s/demods/*/enable' % device, 0])
        general_setting.append(['/%s/scopes/*/enable' % device, 0])

    daq.set(general_setting)

    # Now set the settings relevant to this experiment
    exp_setting = [['/%s/sigins/%d/imp50'          % (device, in_c), 0],
                   ['/%s/sigins/%d/ac'             % (device, in_c), 0],
                   ['/%s/sigins/%d/range'          % (device, in_c), 2*amplitude],
                   ['/%s/demods/%d/order'          % (device, demod_c), 4],
                   ['/%s/demods/%d/timeconstant'   % (device, demod_c), time_constant],
                   ['/%s/demods/%d/rate'           % (device, demod_c), demod_rate],
                   ['/%s/demods/%d/harmonic'       % (device, demod_c), 1],
                   ['/%s/oscs/%d/freq'             % (device, osc_c), 1e6],
                   ['/%s/sigouts/%d/on'            % (device, out_c), 1],
                   ['/%s/sigouts/%d/enables/%d'    % (device, out_c, out_mixer_c), 1],
                   ['/%s/sigouts/%d/range'         % (device, out_c), 1],
                   ['/%s/sigouts/%d/amplitudes/%d' % (device, out_c, out_mixer_c), amplitude]]
    # Device class dependent configuration
    if re.match('HF2', devtype):
        exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
        exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
        exp_setting.append(['/%s/sigins/%d/diff'      % (device, in_c), 0])
        exp_setting.append(['/%s/sigouts/%d/add'      % (device, out_c), 0])
    else:  # UHF
        exp_setting.append(['/%s/demods/%d/enable' % (device, demod_c), 1])
        if re.match('.*MF', options):
            exp_setting.append(['/%s/demods/%d/oscselect' % (device, demod_c), osc_c])
            exp_setting.append(['/%s/demods/%d/adcselect' % (device, demod_c), in_c])
    daq.set(exp_setting)

    # Wait for the demodulator filter to settle
    time.sleep(10*time_constant)

    # Perform a global synchronisation between the device and the data server:
    # Ensure that the settings have taken effect on the device before issuing
    # the getSample() command. Note: the sync() must be issued after waiting for
    # the demodulator filter to settle above.
    daq.sync()

    # Obtain one demodulator sample via ziDAQServer's low-level getSample()
    # method - for extended data acquisition it's preferable to use
    # ziDAQServer's poll() method or the ziDAQRecorder class.
    sample = daq.getSample('/%s/demods/0/sample' % device)
    sample['R'] = np.sqrt(sample['x']**2 + sample['y']**2)
    print 'Measured rms amplitude is %fV.' % sample['R']
    return sample

if __name__ == '__main__':
    demod_sample = run_example()
